"""Top-level package for CoralNet-Toolbox."""

__version__ = "0.0.83"
__author__ = "Jordan Pierce"
__email__ = "jordan.pierce@noaa.gov"
__credits__ = "National Center for Coastal and Ocean Sciences (NCCOS)"
